﻿using System;
using System.Collections.Generic;
using System.Diagnostics;

namespace MapFold
{
    internal class Program
    {
        private static void Main()
        {
            var domains = new List<string>
                              {
                                  "www.google.com",
                                  "www.yahoo.com",
                                  "www.akamai.com",
                                  "www.ask.com"
                              };

            var pinger = new FakePing();
            int pingCount = 20;

            Console.WriteLine("Imperative:");

            var sw = new Stopwatch();
            sw.Start();

            long time = 0;
            foreach (string domain in domains)
            {
                int successfulPingCount = 0;
                for (int i = 0; i < pingCount; i++)
                {
                    FakePingReply reply = pinger.Send(domain);
                    if (reply.RoundtripTime != 0)
                    {
                        successfulPingCount++;
                        time += reply.RoundtripTime;
                    }
                }
                time = time / successfulPingCount;
                Console.WriteLine(domain + ": " + time);
            }
            sw.Stop();

            Console.WriteLine("Elapsed: " + sw.ElapsedMilliseconds);
            sw.Reset();

            #region Using Higher Order Functions

            Console.WriteLine("Functional:");

            sw.Start();

            IEnumerable<IEnumerable<int>> totals =
                domains.ParallelMap(d =>
                                    d.Repeat(domain => new FakePing()
                                                           .Send(domain).RoundtripTime, pingCount)
                                        .Filter(p => p != 0).Apply()
                    );

            IEnumerable<string> averages =
                totals.Map(p =>
                           (p.Fold((p1, p2) => p1 + p2) / p.Count()).ToString()
                    );

            domains.Zip(averages).ForEach(p => Console.WriteLine(p[0] + ": " + p[1]));

            sw.Stop();
            Console.WriteLine("Elapsed: " + sw.ElapsedMilliseconds);

            #endregion
        }
    }
}